/*  common.css - borrow from Redmine and MoinMoin */

/* content styles */
html,body{min-height:100%;}
body{font-size:13px;font-style:normal;color:#4e4e4e;font-family: Verdana, sans-serif; padding: 0;}

/* body with background color */
body {
  text-align:center; /* for IE */
  margin:0 auto; /* for the rest */
  background:#39424b;
}

/* content: align center with fixed width */
div.page
{
  text-align:left;
  width:780px; /* or a percentage, or whatever */
  margin:0 auto 0 auto;
  padding:1em;
  background:#ffffff;
}

/* header and footer: align center with fixed width */
div.header, div.footer
{
  width:780px; /* or a percentage, or whatever */
  margin:0 auto 0 auto;
  padding:1em;
}

div.header {color:#F8F8F8;background-color:#507AAA;text-align:right;font-size:9pt;position:relative;}

div.footer {background-color:#FFFFBB;text-align:right;font-size:9pt;font-style:italic;}

div.footer {text-align:center;}
hr.header, hr.footer {color:#b4aca1;background-color:#b4aca1;border-bottom:1px solid #ffffff;}

.footer a,a:hover,a:link,a:active {color:#0071bb;text-decoration:none;}

/* menu */
#menu {position: absolute;  bottom: 0px;  left:6px; margin-right: -200px;}
#menu ul {margin: 0;  padding: 0;}
#menu li {
  float:left;
  list-style-type:none;
  margin: 0px 2px 0px 0px;
  padding: 0px 0px 0px 0px;
  white-space:nowrap;
}
#menu li a {
  display: block;
  color: #fff;
  text-decoration: none;
  font-weight: bold;
  margin: 0;
  padding: 4px 10px 4px 10px;
}
#menu li a:hover {background:#759FCF; color:#fff;}
#menu li a.selected, #menu li a.selected:hover {background:#fff; color:#555;}

#profile {position: absolute;  bottom: 0px;  right:6px;}
#profile ul {margin: 0;  padding: 0;}
#profile li {
  float:left;
  list-style-type:none;
  margin: 0px 2px 0px 0px;
  padding: 0px 0px 0px 0px;
  white-space:nowrap;
}
#profile li a {
  display: block;
  color: #fff;
  text-decoration: none;
  font-weight: bold;
  margin: 0;
  padding: 4px 10px 4px 10px;
}
#profile li a:hover {background:#759FCF; color:#fff;}
#profile li a.selected, #profile li a.selected:hover {background:#fff; color:#555;}


/* Links */
a{color:#0071bb;outline:none;}
a:hover{color:#ed1e24;text-decoration:none;}

li p {
	margin: .25em 0;
}

li.gap {
    margin-top: 0.5em;
}

dt {
    margin-top: 0.5em;
    font-weight: bold;
}

dd {
    margin-top: 0;
    margin-bottom: 0;
}

dd p {
    margin: 0.25em 0;
}
   
a, img, img.drawing {
	border: 0;
}

pre {
	border: 1pt solid #AEBDCC;
	background-color: #F3F5F7;
	padding: 5pt;
	font-family: courier, monospace;
	white-space: pre;
	/* begin css 3 or browser specific rules - do not remove!
	see: http://forums.techguy.org/archive/index.php/t-249849.html */
    white-space: pre-wrap;
    word-wrap: break-word;
    white-space: -moz-pre-wrap;
    white-space: -pre-wrap;
    white-space: -o-pre-wrap;
    /* end css 3 or browser specific rules */
}

pre.comment {
    background-color: #CCCCCC;
    color: red;
    padding: 0;
    margin: 0;
    border: 0;
}

pre.comment:before {
    content: url(../img/attention.png);
}


/* .comment css definition must be top of .red/.green/.blue or it won't work */
.comment { color: #555555; background-color: #DDDDFF; }

.red { background-color: #FFCCCC; }
.green { background-color: #CCFFCC; }
.blue { background-color: #CCCCFF; }
.yellow { background-color: #FFF29F; }
.orange { background-color: #FFD59B; }

.solid { border: 2px solid #000000; padding: 2px; }
.dashed { border: 2px dashed #000000; padding: 2px; }
.dotted { border: 2px dotted #000000; padding: 2px; }

table
{
	margin: 0.5em 0 0 0.5em;
	border-collapse: collapse;
}

th, td
{
	padding: 0.25em 0.5em 0.25em 0.5em;
	border: 1pt solid #ADB9CC;
}

td p {
	margin: 0;
	padding: 0;
}

table.hidden {
	margin: 0.5em 0 0 0.5em;
	border:  0px;
	border-collapse: collapse;
}

table.hidden th {
    font-weight: bold;
	border: 0px;
}

table.hidden td {
	border: 0px;
}

/* standard rule ---- */
hr {
    height: 1pt;
    background-color: #9C9C9C;
    border: 0;
}

/* eye catchers */
.info {
    float: right;
    font-size: 0.7em;
    color: gray;
}

.warning 
{
	color: red;
}

.error 
{
	color: red;
}

strong.highlight 
{
	background-color: #CCE0FF;
	padding: 1pt;
}

#message_box {
	background-color: #FFD700;
	PADDING: 0.5em;
}

#message .hint {font-style: italic;}
#message .info {
    float: none;
    font-size: 1em;
    color: black;
}
#message .info:before {
    content: url('../img/icon-info.png');
    margin: 0 0.5em 0 0.25em;
    vertical-align: middle;
}
#message .warning:before {
    content: url('../img/alert.png');
    margin: 0 0.5em 0 0.125em;
    vertical-align: middle;
}
#message .error:before {
    content: url('../img/icon-error.png'); 
    margin: 0 0.5em 0 0.125em;
    vertical-align: middle;
}
#message a.clear-link {
    display: block;
    margin: 0.75em 0 0 0;
    font-size: small;
}


